/*
 * The MIT License
 *
 * Copyright 2011 Jacob Dilles.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.gmu.ece445.lc2200;

import com.sun.tools.internal.xjc.util.NullStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Holds a series of assembly instructions
 * 
 * @author Jacob Dilles
 */
public class AssemblyProgram {
    
    public static final boolean DEBUG_PARSING = false;
    /**
     * Pattern for parsing an assembly line
     */
    private static final Pattern LINEREGEX =
            Pattern.compile("^(?:[\\s]*)"
            + "(?:([\\w]{1,16}):)?(?:[\\s]*)" // address label 
            + "([0-9a-fA-F]{4,8})?(?:[\\s]*)" // instruction counter
            + "([a-zA-Z]{2,5})(?:[\\s]*)" // instruction 
            + "([^\\;\\#]+)?" // registers
            + "(?:[\\;\\#]+(.*))?");                 // comments
    int programCntr = -1;
    ArrayList<EncodedInstruction> encodedProgram;
    LinkedHashMap<String, AssemblyInstruction> program;

    
    /**
     * Main method for decoding text input
     * @param instruction
     * @return 
     */
    public static AssemblyInstruction parseInstruction(String line) {
        // Fixes the unicode - used by MS word
        line = line.replace((char) 8211, (char) 45);

        Matcher matcher = LINEREGEX.matcher(line);
        if (!matcher.find()) {
            throw new IllegalArgumentException(line);
        }
        String[] m = new String[matcher.groupCount() + 1];
        if (DEBUG_PARSING) {
            System.out.println("Parsed Line: \"" + line + "\"");
        }
        for (int i = 1; i <= matcher.groupCount(); i++) {
            m[i] = matcher.group(i);
            if (DEBUG_PARSING) {
                System.out.println(i + ": " + m[i]);
            }
        }

        int address = 0;

        if (m[2] != null) {
            address = Integer.decode(m[2]);
        }

        return new AssemblyInstruction(m[1], m[2], m[3], m[4], m[5]);
        

    }
    
    
    
    
    /**
     * Add an instruction manually to the program
     * @param inst 
     */
    public void addInstruction(AssemblyInstruction inst) {
        if (encodedProgram != null) {
            throw new AssertionError("Trying to add instructions after program has been linked");
        }

        if (program == null) {
            program = new LinkedHashMap<String, AssemblyInstruction>(128);
            encodedProgram = null;
            programCntr = -1;
        }

        programCntr++;
        inst.setAddress(programCntr);
        String key = inst.getLabel();
        if (key == null) {
            key = "_pc" + programCntr; // illegal label, could never interfere with a real program
        }
        if ((program.put(key, inst)) != null) {
            throw new RuntimeException("Duplicate label: " + key);
        }

    }
    
    public List<AssemblyInstruction> getProgram(){
        return program == null ? Collections.EMPTY_LIST : new ArrayList<AssemblyInstruction>(program.values());
    }
    
    public List<EncodedInstruction> getEncodedProgram(){
        return encodedProgram == null ? Collections.EMPTY_LIST : new ArrayList<EncodedInstruction>(encodedProgram);
    }

    public int size(){
        return programCntr;
    }
    
    public AssemblyInstruction getInstructionByLabel(String label) {
        return program.get(label);
    }

    public int assembleProgram(PrintStream linkerOutput) {
        if(linkerOutput == null) linkerOutput = new PrintStream(new NullStream());
        if (program == null) {
            throw new AssertionError("Assembling an empty program");
        }
        int counter = 0;
        ArrayList<EncodedInstruction> out = new ArrayList<EncodedInstruction>(programCntr);
        for (AssemblyInstruction inst : program.values()) {
            counter += inst.encode(this,linkerOutput);
            out.add(inst);
        }
        encodedProgram = out;
        return counter;
    }

}
