/*
 * The MIT License
 *
 * Copyright 2011 Jacob Dilles.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.gmu.ece445.lc2200;

import java.util.List;

/**
 *
 * @author Jacob Dilles
 */
public interface LC2200 {

    /**
     * Clock the processor. The instruction in program[pc] is read out, then
     * the PC is incremented.
     *
     * The instruction is then executed. This may affect memory and/or registers
     *
     * @throws RuntimeException if the program has halted due to an error or
     * explicit halt
     * @return true if calling clock() again will execute another instruction
     */
    boolean clock() throws RuntimeException;

    /**
     * Get the instruction which was executed by the LC-2200 the last time that
     * clock() was called
     * @return
     */
    EncodedInstruction getCurrentInstruction();

    /**
     * Get the main memory used by the program. The object is not copied so
     * changing the returned object will affect processor opperation
     * @return
     */
    MainMemory getMemory();

    /**
     * Get a int[n][2] where n is the number of words stored in memory. The
     * format is [[address,value],[address,value],[address,value]]
     * @return
     */
    int[][] getMemoryContents();

    /**
     * Get the current program counter value
     */
    int getPc();

    /**
     * Get a copy of the current register contents. The array has 16 elements.
     * @return
     */
    int[] getRegisterContents();

    /**
     * Get the registers used by the program. The object is not copied so
     * changes to the returned object will be reflected by processor operation
     * @return
     */
    RegisterFile getRegisters();

    /**
     * Set the program counter
     * @param pc
     */
    void setPc(int pc);
    
    /**
     * Get the program loaded into this cpu
     * @return 
     */
    List<EncodedInstruction> getProgram();
 
    /**
     * More detailed report than toString, but not full dump
     * @return 
     */
    String report();
    
    /**
     * Program history (list of program counter values)
     * @return 
     */
    List<Integer> getProgramHistory();
}
