/*
 * The MIT License
 *
 * Copyright 2011 Jacob Dilles.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.gmu.ece445.lc2200;

/**
 *
 * @author Jacob Dilles
 */
public enum OpCode {
    /**
     * 0000  (0)  add  (R): R[Rx] = R[Ry] + R[Rz] 
     */
    add('R'),
    nand('R'),
    addi('I'),
    lw('I'),
    sw('I'),
    beq('I'),
    jalr('J'),
    halt('O'),
    sub('R'),
    nor('R'),
    sll('I'),
    srl('I'),
    sra('I'),
    bne('I'),
    xxx('X'),// because we use ordal 
    nop('X');
    private final char type;
    
    
    private OpCode(char type){
        this.type = type;
    }

    /**
     * Get the instruction type
     * @return 
     */
    public char getType() {
        return type;
    }
    
    /**
     * Get the opcode integer value
     * @return 
     */
    public int getCode(){
        return ordinal();
    }
    
    /**
     * Look up by code number (0-D)
     * @param opcode
     * @return 
     */
    public static OpCode valueOfCode(int opcode){   
        return values()[opcode];
    }
    
    /**
     * Look up by name 
     * @param name
     * @return 
     */
    public static OpCode valueOfName(String name){
        name = name.trim().toLowerCase();
        // Bad documentation
        if("jal".equals(name)){
            name = "jalr";
        }
        return valueOf(name);
    }   
}
