/*
 * The MIT License
 *
 * Copyright 2011 Jacob Dilles.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.gmu.ece445.lc2200;

import java.io.PrintStream;

/**
 * Defines the interface to a register file, used for both the emulator and simulator
 * @author Jacob Dilles
 */
public interface RegisterFile {

    /**
     * Set all registers to 0
     */
    void clear();

    /**
     * Get the contents of the register file. The array has 16 elements
     * @return
     */
    int[] contents();

    /**
     * Dump register contents
     * @param out
     */
    void dump(PrintStream out);

    /**
     * Get a register value
     * @param reg
     * @return
     */
    int get(Register reg);

    /**
     * Set a register value
     * @param register destination
     * @param value new value
     * @return the old value
     */
    int set(Register register, int value);
 
}
