/*
 * The MIT License
 *
 * Copyright 2011 Jacob Dilles.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.gmu.ece445.lc2200;

import java.io.Console;
import java.io.IOException;
import java.io.PrintStream;

/**
 * Some common utilities 
 * @author Jacob Dilles
 */
public class Utils {

    /**
     * Print the given int in binary notation (base 2)
     * @param word
     * @return 
     */
    public static String toBinary(int word) {
        char[] buff = new char[32];
        for (int i = 0; i < 32; i++) {
            buff[i] = (((word >> (31 - i)) & 0x1) == 0) ? '0' : '1';
        }
        return new String(buff);
    }

    /**
     * Print the given int in hex notation (base 16)
     * @param word
     * @return 
     */
    public static String toHex(int word) {
        return new String(new char[]{
                    DIGITS[(int) (0xf & (word >> 28))],
                    DIGITS[(int) (0xf & (word >> 24))],
                    DIGITS[(int) (0xf & (word >> 20))],
                    DIGITS[(int) (0xf & (word >> 16))],
                    DIGITS[(int) (0xf & (word >> 12))],
                    DIGITS[(int) (0xf & (word >> 8))],
                    DIGITS[(int) (0xf & (word >> 4))],
                    DIGITS[(int) (0xf & word)]});
    }

    /**
     * Converts 1-8 unsigned hex characters to an integer
     * @param hex
     * @return 
     */
    public static int fromHexU(String hex) {
        if (hex.isEmpty()) {
            return 0;
        }
        if (hex.length() > 8) {
            throw new IllegalArgumentException("Too long, will overflow int: " + hex);
        }
        int val = 0, temp, i = 0;
        for (i = 0; i < hex.length(); i++) {
            temp = FASTSTR2HEX[hex.charAt(i)];
            if (temp < 0) {
                throw new IllegalArgumentException("Illegal hex character in string [" + hex.charAt(i) + "]: " + hex);
            }
            val <<= 4;
            val |= temp;
        }
        return val;
    }

    /**
     * Arbitrary length signed number hex conversion.
     * e.g. ff01 -> 0xffffff01
     * @param hex
     * @return 
     */
    public static int fromHexS(String hex) {
        int val = fromHexU(hex);
        // sign bit set, number was negative. Set remaining bits to 1
        if (FASTSTR2HEX[hex.charAt(0)] > 7) {
            val |= (0xffffffff << (hex.length() << 2));
        }
        return val;
    }
    private static final char[] DIGITS = "0123456789ABCDEF".toCharArray();
    /**
     * 256 bytes, so all of ascii doesn't throw array out of bounds
     */
    private static final byte[] FASTSTR2HEX = {
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -1, -1, -1, -1, -1,
        -1, 10, 11, 12, 13, 14, 15, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, 10, 11, 12, 13, 14, 15, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
    static Console console = System.console();
    final static int cols = 80;
    final static int rows = 24;
    // Allow redirection 
    private static PrintStream out = System.out;
    /**
     * Gets a single byte integer from console
     * @param message
     * @param defaultInt
     * @return
     */
    public static int getInteger(String message, String defaultInt) {
        int i = -1;
        do {
            try {
                i = Integer.parseInt(getInput(message, defaultInt));
                if (i >= 0 && i < 65536) {
                    return i;
                } else {
                    out.println("Invalid number '" + i + "', Please try agian.");
                }
            } catch (NumberFormatException e) {
                out.println("Invalid entry. Please enter a number.");
            }
        } while (i < 0 || i > 65536);
        return i;
    }
    /**
     * Get an input from the console
     * @param message
     * @param defaultAnswer
     * @return 
     */
    public static String getInput(String message, String defaultAnswer) {
        if (defaultAnswer != null) {
            message += " [" + defaultAnswer + "]";
        }
        message += " : ";
        out.print(message);

        try {
            message = console.readLine();
            /**
             * console.anything() will throw a NPE if we could not get System.console
             * (i.e. java 1.5) that means we need to do it by hand
             */
        } catch (NullPointerException e) {
            message = "";
            int in = -1;
            try {
                // This reads a line off of STD IN
                while ((in = System.in.read()) != -1 && !(message += String.valueOf((char) in)).endsWith("\n"));
            } catch (IOException f) {
                return defaultAnswer;
            }
        }
        if (message == null || message.trim().isEmpty()) {
            return defaultAnswer;
        }
        return message.trim();
    }
//    /**
//     * Converts an array of characters representing hexidecimal values into an
//     * array of bytes of those same values. The returned array will be half the
//     * length of the passed array, as it takes two characters to represent any
//     * given byte. An exception is thrown if the passed char array has an odd
//     * number of elements.
//     *
//     * @param data An array of characters containing hexidecimal digits
//     * @return A byte array containing binary data decoded from
//     *         the supplied char array.
//     */
//    public static byte[] decodeHex(char[] data) {
//        if (data == null) {
//            return null;
//        }
//        int len = data.length;
//
//        // Check minimum length requirement
//        if (len < 2) {
//            return new byte[0];
//        }
//
//        //Hex is 2 chars per byte
//        byte[] out = new byte[len >> 1];
//
//        if ((len & 0x01) != 0) {
//            // try to safely decode the value
//            return decodeHexSafe(data, out);
//        }
//
//        // two characters form the hex value.
//        for (int i = 0, j = 0, k = 0; j < len; i++) {
//            // Look up the value - two chars (one byte) at a time 26 FEB 10 JSD
//            k = ((FASTSTR2HEX[0xff & data[j++]] << 4) | FASTSTR2HEX[0xff & data[j++]]);
//
//            // This should work for all values 0-255 that are illigal, since the
//            // binary "or" opperation on two's complement integer values
//            // carries the negative value around. Clever, eh? JSD
//            if (k < 0) {
//                return decodeHexSafe(data, out);
////                throw new IllegalArgumentException("Illegal hexadecimal at index "
////                        + (j - 2) + ": '" + data[j-2] + "" + data[j-1] +"'");
//            }
//
//            // Set the ouput byte
//            out[i] = (byte) k;
//        }
//
//        return out;
//    }
}
