/*
 * The MIT License
 *
 * Copyright 2011 Jacob Dilles.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.gmu.ece445.lc2200.macros;

import edu.gmu.ece445.lc2200.AssemblyInstruction;
import edu.gmu.ece445.lc2200.AssemblyProgram;
import edu.gmu.ece445.lc2200.OpCode;
import static edu.gmu.ece445.lc2200.OpCode.*;
import static edu.gmu.ece445.lc2200.Register.*;
import edu.gmu.ece445.lc2200.Register;
import java.util.List;

/**
 * 
 * @author Jacob Dilles
 */
public enum Macro {
   
    /**
     * jmp label (1)
     */
    jmp{
        @Override
        void putInstructions(AssemblyInstruction macro, AssemblyProgram program) {
            program.addInstruction(new AssemblyInstruction(beq, zero, zero, macro.getBranchTargetLabel()));
        }
    },
    dec{

        @Override
        void putInstructions(AssemblyInstruction macro, AssemblyProgram program) {
            program.addInstruction(new AssemblyInstruction(addi, macro.getRegisterX(), macro.getRegisterX(), -1));
        }
        
    },
        inc{

        @Override
        void putInstructions(AssemblyInstruction macro, AssemblyProgram program) {
            program.addInstruction(new AssemblyInstruction(addi, macro.getRegisterX(), macro.getRegisterX(), 1));
        }
        
    },
    /**
     * push $rx (2)
     */
    push{

        @Override
        void putInstructions(AssemblyInstruction macro, AssemblyProgram program) {
            program.addInstruction(new AssemblyInstruction(addi, sp, sp, -1));
            program.addInstruction(new AssemblyInstruction(sw, macro.getRegisterX(), sp, 0));
        }
        
    },
    /**
     * pop $rx (2)
     */
    pop{

        @Override
        void putInstructions(AssemblyInstruction macro, AssemblyProgram program) {
            program.addInstruction(new AssemblyInstruction(lw, macro.getRegisterX(), sp, 0));
            program.addInstruction(new AssemblyInstruction(addi, sp, sp, 1));
        }
        
    },
    /**
     * blt $r1, $r2, label (3) if(R[$r1] < R[$r2]) => branch  label 
     * 
     * i.e. $r1 - $r2 < 0
     */
    blt{

        @Override
        void putInstructions(AssemblyInstruction macro, AssemblyProgram program) {
            program.addInstruction(new AssemblyInstruction(sub, at, macro.getRegisterX(), macro.getRegisterY()));
            program.addInstruction(new AssemblyInstruction(sra, at, at, 31));
            program.addInstruction(new AssemblyInstruction(bne, zero, at, macro.getLabel()));
        }
        
    },
    /**
     * blte $r1, $r2, label (4) if(R[$r1] < R[$r2]) => branch  label 
     */
    blte{
                @Override
        void putInstructions(AssemblyInstruction macro, AssemblyProgram program) {
            program.addInstruction(new AssemblyInstruction(beq, macro.getRegisterX(), macro.getRegisterY(), macro.getLabel()));
            program.addInstruction(new AssemblyInstruction(sub, at, macro.getRegisterX(), macro.getRegisterY()));
            program.addInstruction(new AssemblyInstruction(sra, at, at, 31));
            program.addInstruction(new AssemblyInstruction(bne, zero, at, macro.getLabel()));

        }
    }
    
    
    ;
    
    /**
     * Get 
     * @param macro
     * @return 
     */
    abstract void putInstructions(AssemblyInstruction macro, AssemblyProgram program);
    
    
}
